@@ -28,10 +28,9 @@
 #ifndef GCDDB_H
 #define GCDDB_H
 
-void cancel(GtkWidget *widget, GtkWidget **window);
-void gcddb( GtkWidget *button_td );
-void start_cddb();
-int do_cddb( GtkWidget *widget );
+void cancel( GtkWidget *, GtkWidget ** );
+void gcddb( GtkWidget *);
+int do_cddb( GtkWidget *, gpointer );
 
 
 #endif /* GCDDB_H */
